<p class="logo">A <a href="https://www.pocoo.org/">
  <img src="{{ pathto("_static/pocoo.png", 1) }}" alt="Pocoo" /></a>
  {%trans%}project{%endtrans%}</p>

<h3>Download</h3>
<p class="download">{%trans%}Current version: <a href="https://pypi.org/project/Sphinx/" alt="PyPI"><img src="https://img.shields.io/pypi/v/sphinx.svg"></a>{%endtrans%}</p>
<p>{%trans%}Install Sphinx with:{%endtrans%}</p>
<pre>pip install -U Sphinx</pre>

<h3>{%trans%}Questions? Suggestions?{%endtrans%}</h3>

<p>{%trans%}Join the <a href="https://groups.google.com/group/sphinx-users">sphinx-users</a> mailing list on Google Groups:{%endtrans%}</p>
<div class="subscribeformwrapper">
<form action="https://groups.google.com/group/sphinx-users/boxsubscribe"
      class="subscribeform">
  <input type="text" name="email" value="your@email"
         onfocus="$(this).val('');" />
  <input type="submit" name="sub" value="Subscribe" />
</form>
</div>
<p>{%trans%}or come to the <tt>#sphinx-doc</tt> channel on FreeNode.{%endtrans%}</p>
<p>{%trans%}You can also open an issue at the
  <a href="https://github.com/sphinx-doc/sphinx/issues">tracker</a>.{%endtrans%}</p>
