Documentation.addTranslations({"locale": "et", "messages": {"%(filename)s &#8212; %(docstitle)s": "%(filename)s &8212; %(docstitle)s", "&#169; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&#169; <a href=\"%(path)s\">Autori\u00f5igused</a> %(copyright)s.", "&#169; Copyright %(copyright)s.": "&#169; Autori\u00f5igused %(copyright)s.", ", in ": "", "About these documents": "Info selle dokumentatsiooni kohta", "Automatically generated list of changes in version %(version)s": "Automaatselt genereeritud nimekiri versiooni %(version)s muutustest", "C API changes": "C API muutused", "Changes in Version %(version)s &#8212; %(docstitle)s": "Muutused versioonis %(version)s &#8212; %(docstitle)s", "Collapse sidebar": "Varja k\u00fclgriba", "Complete Table of Contents": "T\u00e4ielik sisukorratabel", "Contents": "Sisukord", "Copyright": "Autori\u00f5igus", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Loodud <a href=\"http://sphinx-doc.org/\">Sphinxiga</a> (versioon: %(sphinx_version)s).", "Expand sidebar": "N\u00e4ita k\u00fclgriba", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Selle vormi abil saab otsida k\u00e4esolevast dokumentatsioonist. Sisesta allolevasse lahtrisse otsis\u00f5nad ning kl\u00f5psa \"Otsi\". Pane t\u00e4hele, et otsimisfunktsioon otsib automaatselt lehti, mis sisaldavad k\u00f5iki sisestatud s\u00f5nu. V\u00e4hemate s\u00f5nadega lehed otsingutulemustesse ei ilmu.", "Full index on one page": "T\u00e4isindeks \u00fchel lehel", "General Index": "\u00dcldindeks", "Global Module Index": "Globaalne moodulite indeks", "Go": "Otsi", "Hide Search Matches": "Varja otsingu tulemused", "Index": "Indeks", "Index &ndash; %(key)s": "Indeks &ndash; %(key)s", "Index pages by letter": "Indeksi lehek\u00fcljed algust\u00e4he kaupa", "Indices and tables:": "Indeksid ja tabelid:", "Last updated on %(last_updated)s.": "Viimati uuendatud %(last_updated)s.", "Library changes": "Teegi muutused", "Navigation": "Navigatsioon", "Next topic": "J\u00e4rgmine teema", "Other changes": "\u00dclej\u00e4\u00e4nud muutused", "Overview": "\u00dclevaade", "Permalink to this definition": "P\u00fcsiviit sellele definitsioonile", "Permalink to this headline": "P\u00fcsiviit sellele pealkirjale", "Please activate JavaScript to enable the search\n    functionality.": "Otsingu v\u00f5imaldamiseks tuleb aktiveerida JavaScript.", "Preparing search...": "Otsingu ettevalmistamine...", "Previous topic": "Eelmine teema", "Quick search": "Kiirotsing", "Search": "Otsing", "Search Page": "Otsinguleht", "Search Results": "Otsingu tulemused", "Search finished, found %s page(s) matching the search query.": "Otsingu tulemusena leiti %s leht(e).", "Search within %(docstitle)s": "Otsi %(docstitle)s piires", "Searching": "Otsimine", "Show Source": "N\u00e4ita l\u00e4htekoodi", "Table of Contents": "Sisukorratabel", "This Page": "K\u00e4esolev leht", "Welcome! This is": "Tervitused! See on", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Sinu otsingule ei vastanud \u00fckski dokument. Palun veendu, et k\u00f5ik sisestatud s\u00f5nad on \u00f5igesti kirjutatud ja sa oled valikud piisaval hulgal kategooriaid.", "all functions, classes, terms": "k\u00f5ik funktsioonid, klassid ja terminid", "can be huge": "v\u00f5ib olla v\u00e4ga suur", "last updated": "viimati uuendatud", "lists all sections and subsections": "toob v\u00e4lja k\u00f5ik sektsioonid ja alamsektsioonid", "next chapter": "j\u00e4rgmine jaotis", "previous chapter": "eelmine jaotis", "quick access to all modules": "kiire ligip\u00e4\u00e4s k\u00f5igile moodulitele", "search": "otsi", "search this documentation": "otsi sellest dokumentatsioonist", "the documentation for": "dokumentatsioon projektile"}, "plural_expr": "(n != 1)"});