# Literally included file using Python highlighting

@class_decorator
@other_decorator()
class TheClass(object):

    @method_decorator
    @other_decorator()
    def the_method():
        pass

@function_decorator
@other_decorator()
def the_function():
    pass
